@@ -19,10 +19,6 @@
  * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#if !defined(lint) && !defined(LINT)
-static char rcsid[] = "$Id: crontab.c,v 1.12 2004/01/23 18:56:42 vixie Exp $";
-#endif
-
 /* crontab - install and manage per-user crontab files
  * vix 02may87 [RCS has the rest of the log]
  * vix 26jan87 [original]
@@ -115,7 +111,7 @@ main(int argc, char *argv[]) {
 			"You (%s) are not allowed to use this program (%s)\n",
 			User, ProgramName);
 		fprintf(stderr, "See crontab(1) for more information\n");
-		log_it(RealUser, Pid, "AUTH", "crontab command not allowed");
+		log_it(RealUser, Pid, "AUTH", "crontab command not allowed", 0);
 		exit(ERROR_EXIT);
 	}
 
@@ -293,7 +289,7 @@ list_cmd(void) {
 	FILE *f;
 	int ch;
 
-	log_it(RealUser, Pid, "LIST", User);
+	log_it(RealUser, Pid, "LIST", User, 0);
 	if (!glue_strings(n, sizeof n, SPOOL_DIR, User, '/')) {
 		fprintf(stderr, "path too long\n");
 		exit(ERROR_EXIT);
@@ -326,7 +322,7 @@ delete_cmd(void) {
 	      )	exit(0);
 	}
 
-	log_it(RealUser, Pid, "DELETE", User);
+	log_it(RealUser, Pid, "DELETE", User, 0);
 	if (!glue_strings(n, sizeof n, SPOOL_DIR, User, '/')) {
 		fprintf(stderr, "path too long\n");
 		exit(ERROR_EXIT);
@@ -366,7 +362,7 @@ edit_cmd(void) {
 	PID_T pid, xpid;
 	int uid;
 	
-	log_it(RealUser, Pid, "BEGIN EDIT", User);
+	log_it(RealUser, Pid, "BEGIN EDIT", User, 0);
 	if (!glue_strings(n, sizeof n, SPOOL_DIR, User, '/')) {
 		fprintf(stderr, "path too long\n");
 		exit(ERROR_EXIT);
@@ -607,7 +603,7 @@ edit_cmd(void) {
  remove:
 	unlink(Filename);
  done:
-	log_it(RealUser, Pid, "END EDIT", User);
+	log_it(RealUser, Pid, "END EDIT", User, 0);
 }
 
 /* returns	0	on success
@@ -655,7 +651,7 @@ replace_cmd(void) {
 	 */
 	/*fprintf(tmp, "# DO NOT EDIT THIS FILE - edit the master and reinstall.\n");
 	 *fprintf(tmp, "# (%s installed on %-24.24s)\n", Filename, ctime(&now));
-	 *fprintf(tmp, "# (Cron version %s -- %s)\n", CRON_VERSION, rcsid);
+	 *fprintf(tmp, "# (Cron version %s)\n", CRON_VERSION);
 	 */
 #ifdef WITH_SELINUX
 	if ( selinux_context )
@@ -758,7 +754,7 @@ replace_cmd(void) {
 		goto done;
 	}
 	TempFilename[0] = '\0';
-	log_it(RealUser, Pid, "REPLACE", User);
+	log_it(RealUser, Pid, "REPLACE", User, 0);
 
 	poke_daemon();
 
