@@ -19,20 +19,26 @@
  * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#if !defined(lint) && !defined(LINT)
-static char rcsid[] = "$Id: misc.c,v 1.16 2004/01/23 18:56:43 vixie Exp $";
-#endif
-
 /* vix 26jan87 [RCS has the rest of the log]
  * vix 30dec86 [written]
  */
 
 #include <cron.h>
-#include <limits.h>
 #ifdef WITH_AUDIT
 #include <libaudit.h>
 #endif
 
+#ifdef HAVE_FCNTL_H    /* fcntl(2) */
+# include <fcntl.h>
+#endif
+#ifdef HAVE_UNISTD_H   /* lockf(3) */
+# include <unistd.h>
+#endif
+#ifdef HAVE_FLOCK      /* flock(2) */
+# include <sys/file.h>
+#endif
+#include <stdio.h>
+
 #if defined(SYSLOG) && defined(LOG_FILE)
 # undef LOG_FILE
 #endif
@@ -51,6 +57,25 @@ static int LogFD = ERR;
 static int syslog_open = FALSE;
 #endif
 
+#if defined(HAVE_FCNTL) && defined(F_SETLK)
+static int trylock_file(int fd)
+{
+       struct flock fl;
+
+       memset(&fl, '\0', sizeof(fl));
+       fl.l_type = F_WRLCK;
+       fl.l_whence = SEEK_SET;
+       fl.l_start = 0;
+       fl.l_len = 0;
+
+       return fcntl(fd, F_SETLK, &fl);
+}
+#elif defined(HAVE_LOCKF)
+# define trylock_file(fd)      lockf((fd), F_TLOCK, 0)
+#elif defined(HAVE_FLOCK)
+# define trylock_file(fd)      flock((fd), LOCK_EX|LOCK_NB)
+#endif
+
 /*
  * glue_strings is the overflow-safe equivalent of
  *		sprintf(buffer, "%s%c%s", a, separator, b);
@@ -287,6 +312,7 @@ acquire_daemonlock(int closeflag) {
 	char *ep;
 	long otherpid=-1;
 	ssize_t num, len;
+	pid_t pid = getpid();
 
 	if (closeflag) {
 		/* close stashed fd for child so we don't leak it. */
@@ -301,47 +327,46 @@ acquire_daemonlock(int closeflag) {
 		pidfile = _PATH_CRON_PID;
 		/* Initial mode is 0600 to prevent flock() race/DoS. */
 		if ((fd = open(pidfile, O_RDWR|O_CREAT, 0600)) == -1) {
-			sprintf(buf, "can't open or create %s: %s",
-				pidfile, strerror(errno));
-			fprintf(stderr, "%s: %s\n", ProgramName, buf);
-			log_it("CRON", getpid(), "DEATH", buf);
+			int save_errno = errno;
+			sprintf(buf, "can't open or create %s",
+				pidfile);
+			fprintf(stderr, "%s: %s: %s\n", ProgramName, buf, strerror(save_errno));
+			log_it("CRON", pid, "DEATH", buf, save_errno);
 			exit(ERROR_EXIT);
 		}
 
-		if (flock(fd, LOCK_EX|LOCK_NB) < OK) {
+		if (trylock_file(fd) < OK) {
 			int save_errno = errno;
 
 			bzero(buf, sizeof(buf));
 			if ((num = read(fd, buf, sizeof(buf) - 1)) > 0 &&
 			    (otherpid = strtol(buf, &ep, 10)) > 0 &&
 			    ep != buf && *ep == '\n' && otherpid != LONG_MAX) {
-				sprintf(buf,
-				    "can't lock %s, otherpid may be %ld: %s",
-				    pidfile, otherpid, strerror(save_errno));
+				snprintf(buf, sizeof(buf),
+				    "can't lock %s, otherpid may be %ld",
+				    pidfile, otherpid);
 			} else {
-				sprintf(buf,
-				    "can't lock %s, otherpid unknown: %s",
-				    pidfile, strerror(save_errno));
+				snprintf(buf, sizeof(buf),
+				    "can't lock %s, otherpid unknown",
+				    pidfile);
 			}
-			sprintf(buf, "can't lock %s, otherpid may be %ld: %s",
-				pidfile, otherpid, strerror(save_errno));
-			fprintf(stderr, "%s: %s\n", ProgramName, buf);
-			log_it("CRON", getpid(), "DEATH", buf);
+			fprintf(stderr, "%s: %s: %s\n", ProgramName, buf, strerror(save_errno));
+			log_it("CRON", pid, "DEATH", buf, save_errno);
 			exit(ERROR_EXIT);
 		}
 		(void) fchmod(fd, 0644);
 		(void) fcntl(fd, F_SETFD, 1);
 	}
 
-	sprintf(buf, "%ld\n", (long)getpid());
+	sprintf(buf, "%ld\n", (long)pid);
 	(void) lseek(fd, (off_t)0, SEEK_SET);
 	len =  strlen(buf);
 	if( (num = write(fd, buf, len)) != len )
-	    log_it("CRON", getpid(), "write() failed:", strerror(errno));
+	    log_it("CRON", pid, "ERROR", "write() failed", errno);
 	else
 	{	    
 	    if( ftruncate(fd, num) == -1 )
-		log_it("CRON", getpid(), "ftruncate() failed:", strerror(errno));
+		log_it("CRON", pid, "ERROR", "ftruncate() failed", errno);
 	}
 
 	/* abandon fd even though the file is open. we need to keep
@@ -477,7 +502,7 @@ allowed(const char *username, const char *allow_file, const char *deny_file) {
 		if( ( getuid() == 0 ) && (!isallowed) )
 		{
 		    snprintf(buf,sizeof(buf),"root used -u for user %s not in cron.allow",username);
-		    log_it("crontab",getpid(),"warning",buf);
+		    log_it("crontab", getpid(), "warning", buf, 0);
 		    isallowed = TRUE;
 		}
 	} else if ((fp = fopen(deny_file, "r")) != NULL) {
@@ -486,7 +511,7 @@ allowed(const char *username, const char *allow_file, const char *deny_file) {
 		if( ( getuid() == 0 ) && (!isallowed) )
 		{
 		    snprintf(buf,sizeof(buf),"root used -u for user %s in cron.deny",username);
-		    log_it("crontab",getpid(),"warning",buf);
+		    log_it("crontab", getpid(), "warning", buf, 0);
 		    isallowed = TRUE;
 		}
 	}
@@ -502,7 +527,7 @@ allowed(const char *username, const char *allow_file, const char *deny_file) {
 }
 
 void
-log_it(const char *username, PID_T xpid, const char *event, const char *detail) {
+log_it(const char *username, PID_T xpid, const char *event, const char *detail, int err) {
 #if defined(LOG_FILE) || DEBUGGING
 	PID_T pid = xpid;
 #endif
@@ -542,10 +567,12 @@ log_it(const char *username, PID_T xpid, const char *event, const char *detail)
 	 * everything out in one chunk and this has to be atomically appended
 	 * to the log file.
 	 */
-	snprintf(msg, msg_size, "%s (%02d/%02d-%02d:%02d:%02d-%d) %s (%s)\n",
+	snprintf(msg, msg_size, "%s (%02d/%02d-%02d:%02d:%02d-%d) %s (%s)%s%s\n",
 		username,
 		t->tm_mon+1, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec, pid,
-		event, detail);
+		event, detail,
+		err != 0 ? ": " : "",
+		err != 0 ? strerror(err) : "");
 
 	/* we have to run strlen() because sprintf() returns (char*) on old BSD
 	 */
@@ -569,14 +596,20 @@ log_it(const char *username, PID_T xpid, const char *event, const char *detail)
 		syslog_open = TRUE;		/* assume openlog success */
 	}
 
-	syslog(LOG_INFO, "(%s) %s (%s)", username, event, detail);
+	syslog(err != 0 ? LOG_ERR : LOG_INFO,
+		"(%s) %s (%s)%s%s", username, event, detail,
+		err != 0 ? ": " : "",
+		err != 0 ? strerror(err) : "");
+
 
 #endif /*SYSLOG*/
 
 #if DEBUGGING
 	if (DebugFlags) {
-		fprintf(stderr, "log_it: (%s %ld) %s (%s)\n",
-			username, (long)pid, event, detail);
+		fprintf(stderr, "log_it: (%s %ld) %s (%s)%s%s\n",
+			username, (long)pid, event, detail,
+			err != 0 ? ": " : "",
+			err != 0 ? strerror(err) : "");
 	}
 #endif
 }
@@ -753,7 +786,7 @@ strlens(const char *last, ...) {
  *	clobbers the static storage space used by localtime() and gmtime().
  *	If the local pointer is non-NULL it *must* point to a local copy.
  */
-#ifndef HAVE_TM_GMTOFF
+#ifndef HAVE_STRUCT_TM_TM_GMTOFF
 long get_gmtoff(time_t *clock, struct tm *local)
 {
 	struct tm gmt;
@@ -779,4 +812,4 @@ long get_gmtoff(time_t *clock, struct tm *local)
 
 	return (offset);
 }
-#endif /* HAVE_TM_GMTOFF */
+#endif /* HAVE_STRUCT_TM_TM_GMTOFF */
